@@ -2,8 +2,8 @@ package App::paperback;
 
 use v5.10;
 use strict;
-# use warnings;
-our $VERSION = "1.11";
+use warnings;
+our $VERSION = "1.12";
 
 my ($GinFile, $GpageObjNr, $Groot, $Gpos, $GobjNr, $Gstream, $GoWid, $GoHei);
 my (@Gkids, @Gcounts, @GformBox, @Gobject, @Gparents, @Gto_be_created);
@@ -237,7 +237,7 @@ sub createPageResourceDict {
     $resourceDict .= "/$_ $GpageXObject{$_} 0 R" for sort keys %GpageXObject;
     $resourceDict .= ">>";
   }
-  $resourceDict .= "/ExtGState<<\/Gs0 4 0 R>>";
+  $resourceDict .= "/ExtGState<</Gs0 4 0 R>>";
   return $resourceDict;
 }
 
@@ -396,8 +396,7 @@ sub writeEndNode {
   # Arrange and print the end node:
   $nodeObj  = "${endNode} 0 obj<</Type/Pages/Kids [";
   $nodeObj .= "${_} 0 R " for @{ $Gkids[$si] };
-  $nodeObj .= "]/Count ${Gcounts[$si]}/MediaBox \[0 0 ";
-  $nodeObj .= "${GoWid} ${GoHei}\]>>endobj\n";
+  $nodeObj .= "]/Count ${Gcounts[$si]}/MediaBox [0 0 ${GoWid} ${GoHei}]>>endobj\n";
   $Gobject[$endNode] = $Gpos;
   $Gpos += syswrite $OUT_FILE, $nodeObj;
   return $endNode;
@@ -408,7 +407,7 @@ sub writeEndNode {
 sub calcRotateMatrix {
 ##########################################################
   my $rotate = $_[2];
-  my $str = "1 0 0 1 $_[0] $_[1] cm\n";
+  my $str = "1 0 0 1 ${_[0]} ${_[1]} cm\n";
 
   if ($rotate) {
     my $upperX = 0; my $upperY = 0;
@@ -425,19 +424,33 @@ sub calcRotateMatrix {
 ##########################################################
 sub getRootAndMapGobjects {
 ##########################################################
-  my ( $xref, $tempRoot, $buf );
+  my ( $xref, $tempRoot, $buf, $buf2 );
 
-  sysseek $IN_FILE, -50, 2;
-  sysread $IN_FILE, $buf, 100;
+  sysseek $IN_FILE, -150, 2;
+  sysread $IN_FILE, $buf, 200;
   die "[!] File ${GinFile} is encrypted, cannot be used, aborting"
     if $buf =~ m'Encrypt';
-  if ( $buf =~ m'\bstartxref\s+(\d+)' ) {
+
+  if ($buf =~ m'/Prev\s+(\d+)') { # "Versioned" PDF file (several xref sections)
+    while ($buf =~ m'/Prev\s+(\d+)') {
+      $xref = $1;
+      sysseek $IN_FILE, $xref, 0;
+      sysread $IN_FILE, $buf, 200;
+      # Reading 200 chars is NOT enough. Read thru till we find 1st %%EOF:
+      until ($buf =~ m'%%EOF') {
+        sysread $IN_FILE, $buf2, 200;
+        $buf .= $buf2;
+      }
+    }
+  } elsif ( $buf =~ m'\bstartxref\s+(\d+)' ) { # Non-versioned PDF file
     $xref = $1;
-    # stat[7] = filesize
-    die "[!] Invalid XREF, aborting" if $xref > (stat($GinFile))[7];
-    populateGobjects($xref);
-    $tempRoot = getRootFromXrefSection();
+  } else {
+    $xref = 0;
   }
+  # stat[7] = filesize
+  die "[!] Invalid XREF, aborting" if $xref > (stat($GinFile))[7];
+  populateGobjects($xref);
+  $tempRoot = getRootFromXrefSection();
 
   return 0 unless $tempRoot; # No Root object in ${GinFile}, aborting
 
@@ -457,7 +470,7 @@ sub populateGobjects {
   while ($qty) {
     for (1..$qty) {
       sysread $IN_FILE, $readBytes, 20;
-      $GObjects{$idx} = $1 if $readBytes =~ m'^\s?(\d+) 0+ n';
+      $GObjects{$idx} = $1 if $readBytes =~ m'^\s?(\d{10}) \d{5} n';
       ++$idx;
     }
     ($qty, $idx) = extractXrefSection();
@@ -486,8 +499,7 @@ sub getObject {
 ##########################################################
   my $index = $_[0];
 
-  # Either a multiple %%EOF, versioned PDF, or a non-1.4 PDF
-  return 0 if ! defined $GObjects{$index};
+  return 0 if (! defined $GObjects{$index});   # A non-1.4 PDF
   my $buf;
   my ( $offs, $size ) = @{ $GObjects{$index} };
 
@@ -602,7 +614,8 @@ sub writeRes {
   $elObje =~ m'^(\d+ \d+ obj\s*<<)(.+)(>>\s*stream)'s;
   my $strPos = length($1) + length($2) + length($3);
   my $newPart = "<</Type/XObject/Subtype/Form/FormType 1/Resources ${formRes}"
-    . "/BBox \[ $GformBox[0] $GformBox[1] $GformBox[2] $GformBox[3]\] ${2}";
+    . "/BBox [@{GformBox}] ${2}";
+
   ++$GobjNr;
   $Gobject[$GobjNr] = $Gpos;
   my $reference = $GobjNr;
@@ -849,7 +862,4 @@ into a new PDF file. Input PDF should:
 3. Use page sizes of A5 or A6 or Half Letter or Quarter Letter
 or Half Legal or Quarter Legal;
 
-4. Not be a 'versioned' PDF file (one that has been edited
-after its creation).
-
 =cut
